#!/bin/env python3

import cmd, os, sys
import datetime
from twilio.rest import Client


class SmsChat(cmd.Cmd):
    def __init__(self, nicks: dict):
        super().__init__()
        self.max_message_displayed = 10
        self.account_sid = os.environ["TWILIO_ACCOUNT_SID"]
        self.auth_token = os.environ["TWILIO_AUTH_TOKEN"]
        self.conversation_sid = os.environ["TWILIO_CONVERSATION_SID"]
        self.phone_number = os.environ["TWILIO_PHONE_NUMBER"]
        self.twilio = Client(self.account_sid, self.auth_token)
        self.instance = self.twilio.conversations.conversations.get(
            self.conversation_sid
        ).fetch()
        self.nicks = nicks

    def preloop(self):
        self._refresh()

    def default(self, line):
        # TODO: Confirmation if longer than X ?
        if line == "EOF":
            self.do_quit(None)
        self.instance.messages.create(body=line)

    def emptyline(self):
        self._refresh()

    def do_refresh(self, _args):
        """Refresh the conversation content"""
        print("Force refresh")

    def do_quit(self, _args):
        """Quits the app"""
        sys.exit(0)

    def do_ls(self, _args):
        """Lists participants in the conversation"""
        for p in self.instance.participants.list():
            phone_number = p.messaging_binding["address"]
            print(self.prettify(phone_number))

    def prettify(self, phone_number):
        if phone_number in self.nicks.keys():
            return f"{phone_number} ({self.f(phone_number)})"
        return phone_number

    def do_invite(self, phone_number: str):
        """Invites someone to the conversation"""
        self.instance.participants.create(
            messaging_binding_address=phone_number,
            messaging_binding_proxy_address=self.phone_number,
        )

    def do_kick(self, phone_number: str):
        """Kicks someone from the conversation"""
        victim = next(
            p
            for p in self.instance.participants.list()
            if p.messaging_binding["address"] == phone_number
        )
        victim.delete()
        print(f"Kicked {self.prettify(phone_number)}")

    def _refresh(self):
        self._print_history()

        refresh_date = self.d(datetime.datetime.now())
        self.prompt = f"{refresh_date} [SMS]: "

    def _print_history(self):
        msgs = self.instance.messages.list(
            order="desc", limit=self.max_message_displayed
        )
        longest_date = int(max([len(str(self.d(msg.date_updated))) for msg in msgs]))
        longest_nick = int(max([len(str(self.f(msg.author))) for msg in msgs]))
        for msg in msgs[::-1]:  # reversed
            print(
                f"{str(self.d(msg.date_updated)):<{longest_date}} {self.f(msg.author):>{longest_nick}}| {msg.body}"
            )
        print()

    def f(self, number):
        """Returns the friendly_name of the number, if known"""
        return self.nicks.get(number, number)

    def d(self, datetime):
        return datetime.strftime("%H:%M:%S")


if __name__ == "__main__":
    it = iter(sys.argv[1:])
    nick_dict = dict(zip(it, it))
    print(f"Using nick replace list: {nick_dict}")

    SmsChat(nick_dict).cmdloop()